#include <iostream>
#include <fstream>
#include <stack>
#include <vector>
#include <stdlib.h>

#include "Variant.h"
#include "String.h"

using namespace std;

struct Column
{
    string szColumnName;
    string szDataType;
    string szDescription;
};

class Record
{
    friend class XmlDatabase;
private:
    vector<Column> *pvColumns;
    vector<Variant> vRecord;
public:
    Variant &operator [](char *szColumnName);
    Variant &operator ()(char *szColumnName) {return operator[](szColumnName);}
};

Variant &Record::operator [](char *szColumnName)
{
    int pos=-1;
    int i;

    for(i=0; i<(*pvColumns).size(); i++)
    {
        if ((*pvColumns)[i].szColumnName==szColumnName)
        {
            pos=i;
            break;
        }
    }

    if(pos!=-1)
        return vRecord[pos];
    else
        return Null();
}

class RecordSet
{
private:
    vector<Record*> vpRecord;
public:
    //صnԪصá
    Record operator[](unsigned int nIndex){return *vpRecord[nIndex];}
    //ؼ¼еԪظ
    int GetCount() {return vpRecord.size(); }
    void push(Record* vRecord) { vpRecord.push_back(vRecord);}
    void sort(char* szOrderby, bool byIncrease);
};

void RecordSet::sort(char *szOrderby, bool byIncrease)
{
    Record* vrTemp;
    if(vpRecord.size()==0)return;
    if(byIncrease)
    {
        for(unsigned int i=0; i<vpRecord.size()-1; i++)
        {
            for(unsigned int j=i+1; j<vpRecord.size(); j++)
            {
                if((*vpRecord[j])[szOrderby]<(*vpRecord[i])[szOrderby])
                {
                    vrTemp=vpRecord[j];
                    vpRecord[j]=vpRecord[i];
                    vpRecord[i]=vrTemp;
                }
            }
        }
    }
    else
    {
        for(unsigned int i=0; i<vpRecord.size()-1; i++)
        {
            for(unsigned int j=i+1; j<vpRecord.size(); j++)
            {
                if((*vpRecord[j])[szOrderby]>(*vpRecord[i])[szOrderby])
                {
                    vrTemp=vpRecord[j];
                    vpRecord[j]=vpRecord[i];
                    vpRecord[i]=vrTemp;
                }
            }
        }
    }
}

class XmlDatabase
{
protected:
    vector<Column> vColumns;
    vector<Record> vRecords;
    string szFilename;
    bool bAutoSave;

private:
    //ʽһЩڲú

    bool Compare(char a, char b);
    bool count(bool a, char cOper, bool b);
    char Judge(char *szStatement, unsigned int nIndex);

protected:
    bool Boolean(char *szStatement);
    //ʽ
    bool Condition(char *szCondition, unsigned int nIndex);

public:
    XmlDatabase(char *szFileName=NULL){bAutoSave=false;LoadFromFile(szFileName);}
    virtual ~XmlDatabase(){if(bAutoSave)SaveToFile();}

    //ȡԪظ
    unsigned int size(){return vRecords.size();}
    //ֶ
    bool AddColumn(char *szColumn, char *szType, char *szDescription);
    //ɾֶ
    bool DeleteColumn(char *szColumn);
    //ɾֶ
    bool DeleteColumn(unsigned int nIndex);
    //صnIndexԪصãֱӶ
    Record &operator[](unsigned int nIndex){return vRecords[nIndex];}
    //ݣ
    bool Insert(char *szColumns, char *szData);
    //޸ݣ
    bool Update(char *szColumns, char *szData, char *szCondition);
    //ɾݣ
    bool Delete(char *szCondition);
    //ɾݣ
    bool Delete(unsigned int nIndex);
    //ɸѡݣ
    RecordSet Filter(char *szCondition, char *szOrderby, bool bIncrease=true);
    //ȫ
    bool Sort(char *szColumn, bool bIncrease=true);

    //д
    bool UpdateData(bool bSaveToFile=false){if(bSaveToFile)return SaveToFile();else return LoadFromFile();};
    //Ӵ̶ȡ
    bool LoadFromFile(char *szFileName=NULL);
    //д̡
    bool SaveToFile(char *szFileName=NULL);
};

bool XmlDatabase::AddColumn(char *szColumn, char *szType, char *szDescription)
{
    unsigned int i;
    static string szString("String"), szNumber("Number"), szDateTime("DateTime");
    for(i=0; i<vColumns.size(); i++)
        if(vColumns[i].szColumnName==trim(szColumn))
            return false;


    if(szString!=szType&&szNumber!=szType&&szDateTime!=szType)
        return false;

    Column cNewColumn;
    cNewColumn.szColumnName=trim(szColumn);
    cNewColumn.szDataType=szType;
    cNewColumn.szDescription=szDescription;

    vColumns.push_back(cNewColumn);
    Variant vNull;

    for(i=0; i<vRecords.size(); i++)
        vRecords[i].vRecord.push_back(vNull);

    vColumns.size();
    return true;
}

bool XmlDatabase::DeleteColumn(char *szColumn)
{
    for(unsigned int i=0; i<vColumns.size(); i++)
    {
        if(vColumns[i].szColumnName==szColumn)
            return DeleteColumn(i);
    }

    return false;
}

bool XmlDatabase::DeleteColumn(unsigned int nIndex)
{
    if (nIndex<=vColumns.size())
    {
        vColumns.erase(&vColumns[nIndex]);

        for(unsigned int i=0; i<vRecords.size(); i++)
            vRecords[i].vRecord.erase(&vRecords[i].vRecord[nIndex]);

        return true;
    }
    else return false;
}

bool XmlDatabase::Insert(char *szColumns, char *szData)
{
    vector<string>vClms=Split(szColumns, ",");
    vector<string>vData;

    string szTemp=szData;
    string szBuffer="";
    unsigned int i;

    bool inQuote=false;
    bool inPound=false;
    for(i=0; i<szTemp.size(); i++)
    {
        switch(szTemp[i])
        {
        case '#':
            if(inPound)
                inPound=false;
            else
            {
                if(szBuffer.size())
                    szBuffer+='#';
                else
                    inPound=true;
            }
            break;
        case '\'':
            if(inQuote)
            {
                if(szTemp[i+1]=='\'')
                {
                    szBuffer+='\'';
                    i++;
                }
                else
                    inQuote=false;
            }
            else
                inQuote=true;

            break;
        case ',':
            if(inQuote||inPound)
                szBuffer+=',';
            else
            {
                vData.push_back(trim((char *)szBuffer.c_str()));
                szBuffer="";
            }
            break;
        default:
            szBuffer+=szTemp[i];
        }
    }

    if((szTemp=trim((char *)szBuffer.c_str())).size())
        vData.push_back(szTemp);

    while(vData.size()<vClms.size())
        vData.push_back(string(""));

    Record rRecord;
    rRecord.pvColumns=&vColumns;

    for(i=0; i<vColumns.size(); i++)
    {
        Variant vaTemp;
        for(unsigned int j=0; j<vClms.size(); j++)
        {
            if(trim((char *)vClms[j].c_str())==vColumns[i].szColumnName)
                break;
        }

        if(j<vClms.size())
        {
            if(vColumns[i].szDataType=="String")
                vaTemp=vData[j];
            else if(vColumns[i].szDataType=="DateTime")
                vaTemp=DateTime((char *)vData[j].c_str());
            else if(vColumns[i].szDataType=="Number")
            {
                if(vData[j].size())
                    vaTemp=atof(vData[j].c_str());
                else
                    vaTemp=0.0;
            }
        }

        rRecord.vRecord.push_back(vaTemp);
    }

    vRecords.push_back(rRecord);
    return true;
}

bool XmlDatabase::Delete(unsigned int nIndex)
{
    if (nIndex<=vRecords.size())
    {
        vRecords.erase(&vRecords[nIndex]);
        return true;
    }
    else return false;
}

bool XmlDatabase::Delete(char *szCondition)
{
    bool bTemp=false;
    for(unsigned int i=0; i<vRecords.size(); i++)
    {
        if(Condition(szCondition, i))
        {
            bTemp=Delete(i);
            i--;
        }
    }
    return bTemp;
}

bool XmlDatabase::LoadFromFile(char *szFileName)
{
    string szFile;
    char cBuffer;

    vColumns.clear();
    vRecords.clear();

    if(szFileName==NULL)
        szFile=szFilename;
    else
        szFile=szFileName;

    if(szFile=="")
        return false;

    szFilename=szFile;

    fstream fsStream(szFile.c_str(), ios::in);
    if(!fsStream.good())
        return false;

    string szLable;
    string szText;

    string szColumnsText;
    string szDataText;
    string szColumnsDef;
    string szColumnsType;

    bool inBracket=false;
    int nState=0;

    while(!fsStream.eof())
    {
        fsStream.read(&cBuffer, 1);

        switch(cBuffer)
        {
        case '<':
            inBracket=true;

            if(szLable.substr(0, 7)=="Column ")
                nState=1;
            else if(szLable=="Data")
                nState=2;

            switch(nState)
            {
            case 1:
                if(szLable=="/Column")
                    nState=0;
                else
                {
                    if(szLable.size())
                    {
                        vector<string>vColumns;
                        string sWord;
                        for(unsigned int v=0; v<szLable.size(); v++)
                        {
                            bool inQuote=false;
                            switch(szLable[v])
                            {
                            case '\"':
                                if(inQuote)
                                    inQuote=false;
                                else
                                    inQuote=true;

                                break;
                            case ' ':
                                if(inQuote)
                                    sWord+=szLable[v];
                                else
                                {
                                    if(sWord.size())
                                    {
                                        if(trim((char *)sWord.c_str()).substr(0, 5)=="Name=")
                                            szColumnsDef=Replace((char *)sWord.substr(5).c_str(), "'", "");
                                        else if(trim((char *)sWord.c_str()).substr(0, 5)=="Type=")
                                            szColumnsType=Replace((char *)sWord.substr(5).c_str(), "'", "");
                                    }

                                    sWord="";
                                }

                                break;
                            default:
                                sWord+=szLable[v];
                            }
                        }

                        if(sWord.size())
                        {
                            if(trim((char *)sWord.c_str()).substr(0, 5)=="Name=")
                                szColumnsDef=Replace((char *)sWord.substr(5).c_str(), "'", "");
                            else if(trim((char *)sWord.c_str()).substr(0, 5)=="Type=")
                                szColumnsType=Replace((char *)sWord.substr(5).c_str(), "'", "");
                        }

                        AddColumn((char *)szColumnsDef.c_str(), (char *)szColumnsType.c_str(), (char *)szText.c_str());
                    }
                }
                break;
            case 2:
                if(szLable=="/Data")
                    nState=0;
                else
                {
                    if(szLable=="Record")
                    {
                        nState=3;
                        szText="";
                    }
                }

                break;
            case 3:
                if(szLable=="/Record")
                {
                    if(szColumnsText.size()&&szDataText.size())
                    {
                        szColumnsText=szColumnsText.substr(1);
                        szDataText=szDataText.substr(1);
                        Insert((char *)szColumnsText.c_str(), (char *)szDataText.c_str());
                    }
                    szColumnsText="";
                    szDataText="";
                    nState=2;
                }
            }

            if(nState==3&&szLable.size()&&szLable[0]!='/'&&szText.size())
            {
                szColumnsText=szColumnsText+","+szLable;
                szDataText=szDataText+",'"+Replace((char *)HtmlDecode((char *)szText.c_str()).c_str(), "'", "''")+"'";
            }

            szLable="";
            break;
        case '>':
            inBracket=false;
            szText="";
            break;
        case 10:
            break;
        default:
            if(inBracket)
                szLable+=cBuffer;
            else
            {
                if(szLable[0]!='/')
                    szText+=cBuffer;
            }
        }
    }

    return true;
}

bool XmlDatabase::SaveToFile(char *szFileName)
{
    string szFile;
    unsigned int i;

    if(szFileName==NULL)
        szFile=szFilename;
    else
        szFile=szFileName;

    fstream fsStream(szFile.c_str(), ios::out);
    if(!fsStream.good())
        return false;

    fsStream<<"<?xml version=\"1.0\" encoding=\"gb2312\" ?>"<<endl;
    fsStream<<"<Database>"<<endl;

    fsStream<<"  <Columns>"<<endl;
    for(i=0; i<vColumns.size(); i++)
    {
        fsStream<<"    <Column Name=\""<<vColumns[i].szColumnName<<"\"";
        fsStream<<" Type=\""<<vColumns[i].szDataType<<"\">";
        fsStream<<vColumns[i].szDescription;
        fsStream<<"</Column>"<<endl;
    }
    fsStream<<"  </Columns>"<<endl;

    fsStream<<"  <Data>"<<endl;
    for(i=0; i<vRecords.size(); i++)
    {
        unsigned int j;
        fsStream<<"    <Record>"<<endl;
        for(j=0; j<vColumns.size(); j++)
        {
            fsStream<<"      <"<<vColumns[j].szColumnName<<">";

            if(vRecords[i][(char *)vColumns[j].szColumnName.c_str()].isNull())
                fsStream<<"";
            else if(vColumns[j].szDataType=="String")
                fsStream<<(char *)HtmlEncode((char *)vRecords[i][(char *)vColumns[j].szColumnName.c_str()].toString().c_str()).c_str();
            else if(vColumns[j].szDataType=="Number")
                fsStream<<vRecords[i][(char *)vColumns[j].szColumnName.c_str()].toString().c_str();
            else if(vColumns[j].szDataType=="DateTime")
                fsStream<<"#"<<vRecords[i][(char *)vColumns[j].szColumnName.c_str()].toString().c_str()<<"#";

            fsStream<<"</"<<vColumns[j].szColumnName<<">"<<endl;
        }
        fsStream<<"    </Record>"<<endl;
    }
    fsStream<<"  </Data>"<<endl;

    fsStream<<"</Database>"<<endl;
    return true;
}

bool XmlDatabase::Compare(char a, char b)
{
    if (a=='('&&b=='(') return true;
    if (a=='!'&&(b=='!'||b=='&'||b=='|')) return true;
    if (a=='&'&&(b=='&'||b=='|')) return true;
    if (a=='|'&&b=='|') return true;
    if (a=='|'&&(b=='&'||b=='!')) return false;
    if (a=='&'&&b=='!') return false;
    return true;
}

bool XmlDatabase::count(bool a, char cOper, bool b)
{
    if (cOper=='|') return (a||b);
    if (cOper=='&') return (a&&b);
    return false;
}

bool XmlDatabase::Boolean(char *szStatement)
{
    stack<bool> bData;
    stack<char> cOperate;

    int i=0;
    char cTemp;
    char cOper;
    bool bTemp1, bTemp2;
    while(szStatement[i])
    {
        char nChar=szStatement[i];
        if (nChar=='f') bData.push(false);
        if (nChar=='t') bData.push(true);
        if (nChar=='!'||nChar=='&'||nChar=='|')
        {
            cTemp=NULL;
            if (cOperate.size()>0) cTemp=cOperate.top();
            while (Compare(nChar,cTemp)==false&&cOperate.size()>0)
            {
                cOper=cOperate.top();
                cOperate.pop();
                bTemp1=false;
                if (bData.size()>0)
                {
                    bTemp1=bData.top();
                    bData.pop();
                }
                if (cOper=='&'||cOper=='|')
                {
                    bTemp2=false;
                    if (bData.size()>0)
                    {
                        bTemp2=bData.top();
                        bData.pop();
                    }
                    bTemp2=count(bTemp1, cOper, bTemp2);
                    bData.push(bTemp2);
                }
                if (cOper=='!')
                    bData.push(!bTemp1);
                cTemp=NULL;
                if (cOperate.size()>0) cTemp=cOperate.top();
            }
            cOperate.push(nChar);
        }
        if (nChar=='(')
            cOperate.push(nChar);
        if (nChar==')')
        {
            while (cOperate.top()!='('&&cOperate.size()>0)
            {
                cOper=cOperate.top();
                cOperate.pop();
                bTemp1=false;
                if (bData.size()>0)
                {
                    bTemp1=bData.top();
                    bData.pop();
                }
                if (cOper=='&'||cOper=='|')
                {
                    bTemp2=false;
                    if (bData.size()>0)
                    {
                        bTemp2=bData.top();
                        bData.pop();
                    }
                    bTemp2=count(bTemp1, cOper, bTemp2);
                    bData.push(bTemp2);
                }
                if (cOper=='!')
                    bData.push(!bTemp1);
            }
            cOperate.pop();
        }
        i++;
    }
    while(cOperate.size()>0)
    {
        cOper=cOperate.top();
        cOperate.pop();
        bTemp1=false;
        if (bData.size()>0)
        {
            bTemp1=bData.top();
            bData.pop();
        }
        if (cOper=='&'||cOper=='|')
        {
            bTemp2=false;
            if (bData.size()>0)
            {
                bTemp2=bData.top();
                bData.pop();
            }
            bTemp2=count(bTemp1, cOper, bTemp2);
            bData.push(bTemp2);
        }
        if (cOper=='!')
            bData.push(!bTemp1);
    }
    if (bData.size()==1) return bData.top();
    return true;
}

char XmlDatabase::Judge(char *szStatement, unsigned int nIndex)
{
    string szString=szStatement;

    if(szString=="and")
        return '&';
    else if(szString=="or")
        return '|';
    else if(szString=="not")
        return '!';

    unsigned int i;
    bool bNumber=false;
    string sVariable;
    string sOperator;
    string sNumber;

    for(i=0; i<szString.size(); i++)
    {
        switch(szString[i])
        {
        case '>':
        case '<':
        case '=':
            if(bNumber)
                sNumber+=szString[i];
            else
            {
                sOperator+=szString[i];
                bNumber=true;
            }
            break;
        case '#':
            if(sNumber.size()&&sNumber[0]!='#')
                sNumber+='#';
            break;
        default:
            if(bNumber)
                sNumber+=szString[i];
            else
                sVariable+=szString[i];
        }
    }

    Variant vaLeft, vaRight;
    if((sVariable[0]>='0'&&sVariable[0]<='9')||sVariable[0]=='-'||sVariable[0]=='+')
        vaLeft=sVariable;
    else if(sVariable[0]=='\''||sVariable[0]=='#')
        vaLeft=sVariable.substr(1, sVariable.size()-2);
    else if(sVariable=="Null")
        vaLeft=Null();
    else if(sVariable=="Now")
        vaLeft=Now();
    else
        vaLeft=operator [](nIndex).operator []((char *)sVariable.c_str());

    if((sNumber[0]>='0'&&sNumber[0]<='9')||sNumber[0]=='-'||sNumber[0]=='+')
        vaRight=sNumber;
    else if(sNumber[0]=='\''||sNumber[0]=='#')
        vaRight=sNumber.substr(1, sNumber.size()-2);
    else if(sNumber=="Null")
        vaRight=Null();
    else if(sNumber=="Now")
        vaRight=Now();
    else
        vaRight=operator [](nIndex).operator []((char *)sNumber.c_str());

    if(sOperator==">")
    {
        if(vaLeft>vaRight.toString())
            return 't';
        else
            return 'f';
    }
    else if(sOperator=="<")
    {
        if(vaLeft<vaRight.toString())
            return 't';
        else
            return 'f';
    }
    else if(sOperator==">="||sOperator=="=>")
    {
        if(vaLeft>=vaRight.toString())
            return 't';
        else
            return 'f';
    }
    else if(sOperator=="<="||sOperator=="=<")
    {
        if(vaLeft<=vaRight.toString())
            return 't';
        else
            return 'f';
    }
    else if(sOperator=="="||sOperator=="==")
    {
        if(vaLeft==vaRight.toString())
            return 't';
        else
            return 'f';
    }
    else if(sOperator=="<>"||sOperator=="!=")
    {
        if(vaLeft!=vaRight.toString())
            return 't';
        else
            return 'f';
    }

    return 'f';
}

bool XmlDatabase::Condition(char *szCondition, unsigned int nIndex)
{
    bool inQuote=false;
    string szString=szCondition;
    string szResult="";

    string szTemp="";
    for(unsigned int i=0; i<szString.size(); i++)
    {
        switch(szString[i])
        {
        case '\'':
            if(inQuote)
            {
                if(szString[i+1]=='\'')
                    i++;
                else
                    inQuote=false;
            }
            else
                inQuote=true;

            szTemp+='\'';
            break;
        case ' ':
            if(inQuote)
                szTemp+=szString[i];
            else
            {
                if(szTemp.size())
                    szResult+=Judge((char *)szTemp.c_str(), nIndex);

                szTemp="";
            }
            break;
        case '(':
            if(inQuote)
                szTemp+=szString[i];
            else
            {
                szResult+=szString[i];

                if(szTemp.size())
                    szResult+=Judge((char *)szTemp.c_str(), nIndex);

                szTemp="";
            }
            break;
        case ')':
            if(inQuote)
                szTemp+=szString[i];
            else
            {
                if(szTemp.size())
                    szResult+=Judge((char *)szTemp.c_str(), nIndex);

                szResult+=szString[i];

                szTemp="";
            }
            break;
        default:
            szTemp+=szString[i];
        }
    }

    if(szTemp.size())
        szResult+=Judge((char *)szTemp.c_str(), nIndex);

    return Boolean((char *)szResult.c_str());
}

RecordSet XmlDatabase::Filter(char *szCondition, char *szOrderby, bool bIncrease)
{
    unsigned int i;
    RecordSet rsTemp;
    for(i=0; i<vRecords.size(); i++)
    {
        if(Condition(szCondition, i))
            rsTemp.push(&vRecords[i]);
    }
    rsTemp.sort(szOrderby, bIncrease);
    return rsTemp;
}

bool XmlDatabase::Update(char *szColumns, char *szData, char *szCondition)
{
    vector<string>vClms=Split(szColumns, ",");
    vector<string>vData;

    string szTemp=szData;
    string szBuffer="";
    unsigned int i;

    bool inQuote=false;
    bool inPound=false;
    for(i=0; i<szTemp.size(); i++)
    {
        switch(szTemp[i])
        {
        case '#':
            if(inPound)
                inPound=false;
            else
            {
                if(szBuffer.size())
                    szBuffer+='#';
                else
                    inPound=true;
            }
            break;
        case '\'':
            if(inQuote)
            {
                if(szTemp[i+1]=='\'')
                {
                    szBuffer+='\'';
                    i++;
                }
                else
                    inQuote=false;
            }
            else
                inQuote=true;

            break;
        case ',':
            if(inQuote||inPound)
                szBuffer+=',';
            else
            {
                vData.push_back(trim((char *)szBuffer.c_str()));
                szBuffer="";
            }
            break;
        default:
            szBuffer+=szTemp[i];
        }
    }

    if((szTemp=trim((char *)szBuffer.c_str())).size())
        vData.push_back(szTemp);

    while(vData.size()<vClms.size())
        vData.push_back(string(""));

    Record rRecord;
    rRecord.pvColumns=&vColumns;

    for(i=0; i<vColumns.size(); i++)
    {
        Variant vaTemp;
        for(unsigned int j=0; j<vClms.size(); j++)
        {
            if(trim((char *)vClms[j].c_str())==vColumns[i].szColumnName)
                break;
        }

        if(j<vClms.size())
        {
            if(vColumns[i].szDataType=="String")
                vaTemp=vData[j];
            else if(vColumns[i].szDataType=="DateTime")
                vaTemp=DateTime((char *)vData[j].c_str());
            else if(vColumns[i].szDataType=="Number")
            {
                if(vData[j].size())
                    vaTemp=atof(vData[j].c_str());
                else
                    vaTemp=0.0;
            }
        }

        rRecord.vRecord.push_back(vaTemp);
    }

    for(i=0; i<vRecords.size(); i++)
    {
        if(Condition(szCondition, i))
        {
            for(unsigned int j=0; j<rRecord.vRecord.size(); j++)
            {
                if(!rRecord.vRecord[j].isNull())
                    vRecords[i].vRecord[j]=rRecord.vRecord[j];
            }
        }
    }
    return true;
}
